# make climatologies for evaluation data

library(eurocordexr)
library(data.table)
library(magrittr)

dat_ym <- rbind(data.table(year = 2002, month = 10:12),
                CJ(year = 2003:2007, month = 1:12),
                data.table(year = 2008, month = 1:9))

## read all ####

# MODIS
dat_modis <- readRDS("data/modis/03-modis-snc-monthly.rds")
dat_modis_eval <- merge(dat_ym, dat_modis)

# dat_modis_eval[ , .N, .(icell, month)] %>% summary
# dat_modis_eval[ , .N, .(icell, month)] %>%
#   .[N < 6, .N, icell]

dat_modis_eval_clim <- dat_modis_eval[, 
                                      .(snc_mean = mean(snc_mean),
                                        n_years = .N), 
                                      .(icell, month)]

# RCM
dat_rcm <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/grid/01-evaluation-monthly.rds")
dat_rcm[, year := year(date)]
dat_rcm[, month := month(date)]
dat_rcm_eval <- merge(dat_ym, dat_rcm, by = c("year", "month"))

# subset to plausible
dat_th <- readRDS("data/auxilary-03-snow-max-threshold.rds")
dat_rcm_eval <- merge(dat_rcm_eval, dat_th[snow_plausible == "yes", .(rcm, icell)], 
                      by = c("rcm", "icell"))

dat_rcm_eval_clim <- dat_rcm_eval[,
                                  .(snc_mean = mean(snc),
                                    tas_mean = mean(tas),
                                    pr_mean = mean(pr)),
                                  .(gcm, rcm, experiment, icell, month)]


# EOBS
dat_eobs_tg <- nc_grid_to_dt("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/monthly/eobs/tg.nc", "tg")
dat_eobs_rr <- nc_grid_to_dt("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/monthly/eobs/rr.nc", "rr")
dat_eobs <- merge(dat_eobs_tg, dat_eobs_rr)
dat_eobs[, year := year(date)]
dat_eobs[, month := month(date)]
dat_eobs_eval <- merge(dat_ym, dat_eobs, by = c("year", "month"))

dat_eobs_eval_clim <- dat_eobs_eval[,
                                  .(tg_mean = mean(tg),
                                    rr_mean = mean(rr)),
                                  .(icell, month)]

# OROG
dat_orog_modis <- readRDS("data/auxilary-01-modis-altzonealpconv.rds")
dat_orog_eobs <- nc_grid_to_dt("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/fixed/eobs/elev.nc", "elevation")
dat_orog_rcm <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/grid/00-orog.rds")


## merge all ####

# first all orog
dat_orog_modis[, .(icell, x, y, zone, alpconv, alt_modis = alt)] %>% 
  merge(dat_orog_eobs[, .(icell, alt_eobs = elevation)], by = "icell") %>% 
  merge(dat_orog_rcm[, .(rcm, icell, alt_rcm = orog)], by = "icell") -> dat_orog

dat_modis_eval_clim[, .(icell, month, snc_modis = snc_mean, n_years_modis = n_years)] %>% 
  merge(dat_eobs_eval_clim[, .(icell, month, tmean_eobs = tg_mean, pre_eobs = rr_mean)], 
        by = c("icell", "month")) %>% 
  merge(dat_rcm_eval_clim[, .(gcm, rcm, experiment, icell, month, 
                              snc_rcm = snc_mean, tmean_rcm = tas_mean, pre_rcm = pr_mean)],
        by = c("icell", "month")) %>% 
  merge(dat_orog, by = c("rcm", "icell")) -> dat_merge

# dat_merge[, .(alt_modis, x, y)] %>% unique %>% ggplot(aes(x,y,fill=alt_modis))+geom_raster()



## add delta's ####

dat_merge[, delta_snc_rcm_modis := snc_rcm - snc_modis]
dat_merge[, delta_tmean_rcm_eobs := tmean_rcm - tmean_eobs]
dat_merge[, delta_pre_rcm_eobs := pre_rcm - pre_eobs]
dat_merge[, delta_alt_rcm_eobs := alt_rcm - alt_eobs]
dat_merge[, delta_alt_rcm_modis := alt_rcm - alt_modis]


## adjust tmean with lapse rate ####

# monthly lapse rates as in liston and elder 2006
# dt_month_lapse <- data.table(month = 1:12,
#                              lapse_rate = c(4.4, 5.9, 7.1, 7.8, 8.1, 8.2, 8.1, 8.1, 7.7, 6.8, 5.5, 4.7))


# monthly lapse rates as in rolland 2003
dt_month_lapse <- data.table(month = 1:12,
                             lapse_rate = c(4.5, 5.0, 5.8, 6.2, 6.5, 6.5, 6.5, 6.5, 6.0, 5.5, 5.0, 4.5))

dat_merge <- merge(dat_merge, dt_month_lapse, by = "month")

dat_merge[, delta_tmean_rcm_eobs_lapse := 
            delta_tmean_rcm_eobs + delta_alt_rcm_eobs * lapse_rate / 1000]

dat_merge[, delta_tmean_rcm_eobs_modis_lapse := 
            delta_tmean_rcm_eobs_lapse + delta_alt_rcm_modis * lapse_rate / 1000]

dat_merge[, tmean_eobs_lapse := 
            tmean_eobs + delta_alt_rcm_eobs * lapse_rate / 1000]


## save ####

# some na values for eobs -> remove

dat_merge <- na.omit(dat_merge)

saveRDS(dat_merge, file = "data/climatology-01-eval.rds")

## EOF ####