# merge and monthly station RCM data 

# adapted from klimareport-repro

## -------------------------------------------- ##
## PARAMETERS ####
## -------------------------------------------- ##


## -------------------------------------------- ##
## INPUT ####
## -------------------------------------------- ##

library(foreach)
library(data.table)
library(magrittr)

mitmatmisc::init_parallel_ubuntu()

path_in <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/point/01-raw/"
path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/point/02-merged/"


## -------------------------------------------- ##
## OUTPUT ####
## -------------------------------------------- ##

# *.rds files in path_out of the following structure
# : path_out/[station].rds

# each file contains a data.table of the following example:
# [additional columns might contain additional variables]

# Classes ‘data.table’ and 'data.frame':	1410209 obs. of  7 variables:
# $ date      : Date, format: "1949-12-01" "1949-12-01" "1949-12-01" "1949-12-02" ...
# $ gcm       : chr  "ICHEC-EC-EARTH" "MOHC-HadGEM2-ES" "MPI-M-MPI-ESM-LR" "ICHEC-EC-EARTH" ...
# $ rcm       : chr  "CLMcom-CCLM4-8-17" "CLMcom-CCLM4-8-17" "CLMcom-CCLM4-8-17" "CLMcom-CCLM4-8-17" ...
# $ experiment: chr  "historical" "historical" "historical" "historical" ...
# $ pr        : num  0.00 1.72e-08 1.26 0.00 7.81e-03 ...
# $ tasmax    : num  -3.4 -7.28 -3.01 -2.24 -9.33 ...
# $ tasmin    : num  -20.83 -14.7 -7.98 -11.88 -15.52 ...




## -------------------------------------------- ##
## read and combine data ####
## -------------------------------------------- ##


# main dir --- stn       --- var   --- *.rds
#              rbindlist --- merge --- rbindlist

foreach(
  i_dir_stn = list.dirs(path_in, recursive = F)
) %dopar% {
  
  i_stn <- basename(i_dir_stn)
  
  file_to_save <- file.path(path_out, 
                            paste0(i_stn, ".rds"))
  if(!dir.exists(dirname(file_to_save))) dir.create(dirname(file_to_save), recursive = T)
  if(file.exists(file_to_save)) return(paste0("Skipping because file already exists: ", i_stn))
  
  dat_full_stn <- foreach(
    i_dir_stn_var = list.dirs(i_dir_stn, recursive = F),
    .final = function(l) Reduce(function(x,y) merge(x, y, all = T), l)
  ) %do% {
    
    foreach(
      fn = list.files(i_dir_stn_var, full.names = T, recursive = F),
      .final = rbindlist
    ) %do% {
      
      dat <- readRDS(fn)
      
      dat <- dat[, lapply(.SD, mean), .(year(date), month(date))]
      
      fn %>% 
        basename %>% 
        gsub(".rds", "", .) %>% 
        strsplit("_") %>% 
        .[[1]] -> fn_split
      
      dat[, ":="(gcm = fn_split[1],
                 rcm = fn_split[2],
                 experiment = fn_split[3])]
      
      dat
      
    }
    
  }
  
  ## -------------------------------------------- ##
  ## adjust units ####
  ## -------------------------------------------- ##
  
  # pr: kg m-2 s-1 == mm/s --> mm/day
  if("pr" %in% names(dat_full_stn)){
    dat_full_stn[, ":="(pr = pr * 24*60*60)]
  }
  
  # tas*: K --> C 
  temp_vars <- c("tas", "tasmin", "tasmax") 
  temp_vars_in <- temp_vars[temp_vars %in% names(dat_full_stn)]
  if(length(temp_vars_in) > 0){
    dat_full_stn[, 
                 c(temp_vars_in) := lapply(.SD, function(x) x - 273.15), 
                 .SDcols = temp_vars_in]
  }
  
  # aladin53 rcp* is in deg C, so redo step before (but not historical or evaluation)
  if(nrow(dat_full_stn[rcm == "CNRM-ALADIN53" & startsWith(experiment, "rcp")]) > 0 &
     length(temp_vars_in) > 0){
    dat_full_stn[rcm == "CNRM-ALADIN53" & startsWith(experiment, "rcp"), 
                 c(temp_vars_in) := lapply(.SD, function(x) x + 273.15), 
                 .SDcols = temp_vars_in]
  }
  
  
  saveRDS(dat_full_stn, file = file_to_save)
  return(paste0("Done: ", i_stn))
  
}
