# compare the different BC routines


library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(ggplot2)
library(forcats)

# prep data maps (~30min) ---------------------------------------------------------------

dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")
clim_modis <- dat_modis[, 
                        .(snc = mean(snc, na.rm = T),
                          snc_q10 = quantile(snc, 0.1, na.rm = T),
                          snc_q90 = quantile(snc, 0.9, na.rm = T)), 
                        .(icell, month(date))]

dat_delta <- foreach(
  i_fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/01_delta_change/"),
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat[, ":="(rcm = fn_info[1],
             gcm = fn_info[2],
             experiment = fn_info[3])]
  
  dat
  
}

dat_delta2 <- merge(dat_delta, clim_modis, by = c("icell", "month"))
dat_delta2 %>% summary
dat_delta2[is.na(delta_snc_mult), delta_snc_mult := 1] # snc_past == snc_future == 0 -> make 1 (no change)
dat_delta2[delta_snc_mult > 1] %>% summary
dat_delta2[, snc_future_delta := snc * delta_snc_mult]
dat_delta2[snc_future_delta > 1, snc_future_delta := 1]

dat_delta2[, snc_q10_future_delta := snc_q10 * delta_snc_mult]
dat_delta2[snc_q10_future_delta > 1, snc_q10_future_delta := 1]
dat_delta2[, snc_q90_future_delta := snc_q90 * delta_snc_mult]
dat_delta2[snc_q90_future_delta > 1, snc_q90_future_delta := 1]

# QM
clim_qm <- foreach(
  i_fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/02_qm/"),
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat_clim <- dat[,
                  .(snc = mean(snc),
                    snc_q10 = quantile(snc, 0.1),
                    snc_q90 = quantile(snc, 0.9),
                    snc_qm = mean(snc_qm),
                    snc_qm_q10 = quantile(snc_qm, 0.1),
                    snc_qm_q90 = quantile(snc_qm, 0.9)),
                  .(icell, month)]
  
  dat_clim[, ":="(rcm = fn_info[1],
                  gcm = fn_info[2],
                  experiment = fn_info[3],
                  fp = fn_info[6])]
  
  dat_clim
  
}


# QDM
clim_qdm <- foreach(
  i_fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/03_qdm/"),
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat_clim <- dat[,
                  .(snc = mean(snc),
                    snc_q10 = quantile(snc, 0.1),
                    snc_q90 = quantile(snc, 0.9),
                    snc_qdm = mean(snc_qdm),
                    snc_qdm_q10 = quantile(snc_qdm, 0.1),
                    snc_qdm_q90 = quantile(snc_qdm, 0.9)),
                  .(icell, month)]
  
  dat_clim[, ":="(rcm = fn_info[1],
                  gcm = fn_info[2],
                  experiment = fn_info[3],
                  fp = fn_info[6])]
  
  dat_clim
  
}


# QDM
clim_mbc <- foreach(
  i_fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/04_mbc_qdm/"),
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat_clim <- dat[!is.na(snc_qdm),
                  .(snc = mean(snc),
                    snc_q10 = quantile(snc, 0.1),
                    snc_q90 = quantile(snc, 0.9),
                    snc_qdm = mean(snc_qdm),
                    snc_qdm_q10 = quantile(snc_qdm, 0.1),
                    snc_qdm_q90 = quantile(snc_qdm, 0.9)),
                  .(icell, month)]
  
  dat_clim[, ":="(rcm = fn_info[1],
                  gcm = fn_info[2],
                  experiment = fn_info[3],
                  fp = fn_info[6])]
  
  dat_clim
  
}


# save clim ---------------------------------------------------------------

save(clim_modis, dat_delta2, clim_qm, clim_qdm, clim_mbc,
     file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/clim-01.rda")



# prep data ecdf (6 min) -------------------------------------------------


icell_sample <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/sample-pixels-01-stratified-elev.rds")

# QM
ecdf_qm <- foreach(
  i_fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/02_qm/"),
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat <- dat[icell %in% icell_sample]
  
  dat[, ":="(rcm = fn_info[1],
             gcm = fn_info[2],
             experiment = fn_info[3],
             fp = fn_info[6])]
  
  dat
  
}


# QDM
ecdf_qdm <- foreach(
  i_fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/03_qdm/"),
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat <- dat[icell %in% icell_sample]
  
  dat[, ":="(rcm = fn_info[1],
             gcm = fn_info[2],
             experiment = fn_info[3],
             fp = fn_info[6])]
  
  dat
  
}

# MBC
ecdf_mbc <- foreach(
  i_fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/04_mbc_qdm/"),
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat <- dat[icell %in% icell_sample]
  
  dat[, ":="(rcm = fn_info[1],
             gcm = fn_info[2],
             experiment = fn_info[3],
             fp = fn_info[6])]
  
  dat
  
}

# delta change
dat_modis[, month := month(date)]
dat_delta[icell %in% icell_sample] %>% 
  merge(dat_modis, by = c("icell", "month")) -> ecdf_delta
ecdf_delta %>% summary

ecdf_delta[is.na(delta_snc_mult), delta_snc_mult := 1] # snc_past == snc_future == 0 -> make 1 (no change)
ecdf_delta[delta_snc_mult > 1] %>% summary
ecdf_delta[, snc_future_delta := snc * delta_snc_mult]
ecdf_delta[snc_future_delta > 1, snc_future_delta := 1]




# save ecdf ---------------------------------------------------------------

save(ecdf_qm, ecdf_qdm, ecdf_mbc, ecdf_delta,
     file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_comparison/ecdf-01.rda")
