# implement filling of scf_th based on same altitude from similar lr-pixels

# search up to 50 similar pixels 

library(raster)
library(data.table)
library(magrittr)
library(foreach)

source("R/functions/cp_helpers.R")
source("R/functions/cp_mae.R")


# settings ----------------------------------------------------------------

n_sim_pixels <- 50 # number of lr pixels to look for
alt_buffer <- 150 # meters to average over scf th
n_hr_pixels_mean <- 100 # number of hr pixels (ordered by lr similarity) to avg scf_th


# prep data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later

rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v03_imputed_similar.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      water = rr_water[],
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]




# add alt_diff
rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


# remove water and not available pixels
dat_scf_eval <- dat_scf[!is.na(alt) & !is.na(alt_lr) & water == 0]


# prep for loop 
dat_scf_eval_missing <- dat_scf_eval[, 
                                     .(n_na = sum(is.na(scf_th)),
                                       n_avail = sum(!is.na(scf_th))),
                                     .(icell_lr)]

lr_icell <- dat_scf_eval_missing[n_na > 0, icell_lr]

# dist lr icells
mat_dist <- as.matrix(readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-01-wasserstein.rds"))



# main loop ---------------------------------------------------------------

mitmatmisc::init_parallel_ubuntu(12)

dat_icell_impute <- foreach(
  i_lr_icell = lr_icell,
  # i_lr_icell = lr_icell[c(1:10, 13:26)], # test n_na>=10
  # i_lr_icell = lr_icell[c(1:24)],
  .inorder = F,
  .final = rbindlist
) %dopar% {
  
  dt_wasserdist_i <- icell_min_dist(i_lr_icell, n_sim_pixels)
  
  i_lr_icell_similar <- icell_min_dist(i_lr_icell, n_sim_pixels)$icell_with_min_dist
  
  i_hr_missing <- dat_scf_eval[icell_lr == i_lr_icell & is.na(scf_th), icell]
  
  dat_scf_eval_i <- dat_scf_eval[!is.na(scf_th) & 
                         icell_lr %in% c(i_lr_icell, i_lr_icell_similar)]
  
  lapply(i_hr_missing, function(x){
    
    x_alt <- dat_scf_eval[icell == x, alt]
    
    # select only specific number of best pixels
    dt_scf_eval_i_alt <- dat_scf_eval_i[between(alt, 
                                               x_alt - alt_buffer, 
                                               x_alt + alt_buffer)]
    if(nrow(dt_scf_eval_i_alt) == 0) return(data.table(scf_th = NA_real_,
                                                       nn = 0,
                                                       icell_orig = x))
    
    dt_scf_eval_i_alt2 <- merge(dt_scf_eval_i_alt, dt_wasserdist_i, 
                                by.x = "icell_lr", by.y = "icell_with_min_dist")
    setorder(dt_scf_eval_i_alt2, min_dist)
    dt_scf_impute <- dt_scf_eval_i_alt2[1:min(n_hr_pixels_mean, .N), 
                                        .(scf_th = mean(scf_th),
                                          nn = .N)]
    
    # based on all 50 similar icell (might be much off)
    # dt_scf_impute <- dat_scf_eval_i[between(alt, 
    #                                         x_alt - alt_buffer, 
    #                                         x_alt + alt_buffer), 
    #                                 .(scf_th = mean(scf_th),
    #                                   nn = .N)]
    dt_scf_impute[, icell_orig := x]
    dt_scf_impute
    
  }) %>% rbindlist -> dat_out
  
  dat_out
  
}



# save --------------------------------------------------------------------

# icells with imputed scf
saveRDS(dat_icell_impute, 
        "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-03-impute-scf-alt.rds")

rr_scf_new <- raster(rr_scf)
rr_scf_new[dat_icell_impute$icell_orig] <- dat_icell_impute$scf_th

rr_scf_new[dat_scf_eval[!is.na(scf_th), icell]] <- dat_scf_eval[!is.na(scf_th), scf_th]

writeRaster(rr_scf_new, 
            "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif",
            overwrite = T)

# mae imputed
# -> (can be calculated from imputed cell data)


# wasser_dist of next icell 
# -> (can be computed from imputed cell data)



# EOF ---------------------------------------------------------------------


