# plot time series of SNC changes
# BC data 0.11

library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(ggplot2)
library(forcats)


load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
rcm_short <- readRDS("data/auxiliary-01-rcm-names.rds")

dat_aux <- readRDS("data/auxiliary-03-modis.rds")

# alpine wide avg ---------------------------------------------------------

# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         !rcm_short %in% c("WRF331F", "HIRHAM5", "RACMO22E")] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)


# add
clim_modis[, season := mitmatmisc::season_fct(month)]
clim_qdm[, season := mitmatmisc::season_fct(month)]

dat_aux[icell %in% icell_common, alt] %>% qplot(binwidth = 100)
dat_aux[, alt_f := cut(alt, breaks = c(-10, seq(200, 2600, by = 200), 3000), dig.lab = 5)]

clim_modis2 <- merge(clim_modis[icell %in% icell_common], 
                    dat_aux, by = "icell")
clim_qdm2 <- merge(clim_qdm[icell %in% icell_common], 
                  dat_aux, by = "icell")


dt_modis <- clim_modis2[season %in% c("DJF", "MAM"), 
                       .(snc_mean = mean(snc),
                         snc_q10 = mean(snc_q10),
                         snc_q90 = mean(snc_q90)),
                       .(alt_f, season)]

dt_qdm <- clim_qdm2[season %in% c("DJF", "MAM"), 
                   .(snc_mean = mean(snc_qdm),
                     snc_q10 = mean(snc_qdm_q10),
                     snc_q90 = mean(snc_qdm_q90)), 
                   .(alt_f, season, rcm, gcm, experiment, fp)]


dt_qdm[, rcm_short := rcm_short[rcm]]

# WRF331F not anymore on ESGF
# RACMO has so many "bad" cells
dt_qdm <- dt_qdm[is.na(rcm_short) |
                       !rcm_short %in% c("WRF331F", "RACMO22E")] 


# plot -------------------------------------------------------------

dt_modis %>% 
  ggplot(aes(snc_mean, alt_f))+
  geom_point()+
  facet_wrap(~season)



# dat_plot <- dt_qdm[!(fp == "past" & experiment == "rcp26")]
dat_plot <- dt_qdm[fp == "future"]
dat_plot[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]
dat_plot[, lbl_time := ifelse(fp == "past", 
                              "2000-2020",
                              paste0("2071-2100: ", experiment_f))]
# dat_plot[, lbl_bc := factor(toupper(ff), levels = toupper(c("raw", "dc", "qm", "qdm")))]



dat_plot2 <- melt(dat_plot, measure.vars = c("snc_mean", "snc_q10", "snc_q90"))
dt_modis2 <- melt(dt_modis, measure.vars = c("snc_mean", "snc_q10", "snc_q90"))

dat_plot2[, variable_f := fct_recode(factor(variable, 
                                            levels = c("snc_q10", "snc_mean", "snc_q90")),
                                     "q10" = "snc_q10",
                                     "mean" = "snc_mean",
                                     "q90" = "snc_q90")]
dt_modis2[, variable_f := fct_recode(factor(variable, 
                                            levels = c("snc_q10", "snc_mean", "snc_q90")),
                                     "q10" = "snc_q10",
                                     "mean" = "snc_mean",
                                     "q90" = "snc_q90")]

gg <- dat_plot2 %>% droplevels %>% 
  ggplot(aes(value, alt_f))+
  geom_point(data = droplevels(dt_modis2), aes(shape = "MODIS"))+
  geom_line(aes(colour = rcm_short, linetype = gcm, group = paste0(rcm,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  scale_shape("2000-2020 Obs.")+
  facet_grid(lbl_time + season ~ variable_f)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Snow cover fraction [0-1]")+
  ylab("Elevation [m]")



ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-elev.png",
       gg, 
       width = 10,
       height = 11)



# plot diff ---------------------------------------------------------------

# diff to modis

dat_plot2_diff <- dat_plot2 %>%
  merge(dt_modis2[, .(season, alt_f, variable_f, value_ref = value)])

# or diff to past?
# dat_plot_diff <- dat_plot[fp == "future"] %>%
#   merge(dat_plot[fp == "past", .(ff, month_fct, rcm, gcm, snc_ref = snc)])

dat_plot2_diff[, value_diff := value - value_ref]
dat_plot2_diff[, value_ratio := value / value_ref]

gg <- dat_plot2_diff %>%
  ggplot(aes(alt_f, value_diff))+
  geom_line(aes(colour = rcm_short, linetype = gcm, group = paste0(rcm,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  scale_y_continuous(breaks = seq(-1, 1, by = 0.2))+
  facet_grid(experiment_f + season ~ variable_f, scales = "free_y", space = "free_y")+
  theme_bw()+
  theme(panel.grid.minor = element_blank(),
        axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5))+
  ylab("Absolute change in snow cover fraction")+
  xlab("Elevation [m]")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-elev-diff.png",
       gg, 
       width = 10,
       height = 7)

